
//
// Displays a car at the origin. The purpose of this program is to demonstrate
//    navigation capabilities - "fly through"  or "Maya controls - tumble, dolly, track.

#include "Angel.h"
#include <iostream>
using std::cout;

#include "MatrixStack.h"
#include "Shapes.h"
#include "Car.h"
#include "Axes.h"
#include "Globals.h"
#include "Camera.h"


//********  These are available as extern variables in Globals.h **************
GLuint  projection; // projection matrix uniform shader variable location
GLuint  model_view;   // model-view matrix uniform shader variable location
GLuint  model_color;  // uniform shader variable location for color
GLuint program;  // shader programs
MatrixStack mvMatrixStack;  // stores the model view matrix stack
Shapes shapes;
//********  End extern variables in Globals.h **************

Camera camera;

float animateAngle = 0;  // used for animation

Car myCar;

Axes axes;

//---------------------------------------------------------------------------- printControls

void printControls()
{
    cout << "\n\n************  Controls **************" << "\n";
    cout << "q or Q ............ quit" << "\n";
    cout << "f ................. move car forward" << "\n";
    cout << "b ................. move car backward" << "\n";
    camera.printCameraControls();
}

//---------------------------------------------------------------------------- init
// OpenGL initialization
void
init()
{
    camera.calcUVN();

    program = InitShader( "vertex.glsl", "fragment.glsl" );
    glUseProgram(program );

    // Uniform variables
    model_color = glGetUniformLocation( program, "model_color" );
    model_view  = glGetUniformLocation( program, "model_view" );
    projection  = glGetUniformLocation( program, "projection" );

    glLineWidth(2);  // sets the thickness of the line for the wired shapes
    shapes.createBuffers(program);

    glEnable( GL_DEPTH_TEST );
    glClearColor( 0.5, 0.5, 0.5, 1.0 );
    printControls();
}

//---------------------------------------------------------------------------- drawGround

void drawGround(mat4 mv)
{
// Draw the ground
    mvMatrixStack.pushMatrix(mv);
    mv = mv*Translate(0,-.1,0);
    mv = mv*Scale(30,.2,30);
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv );
    shapes.drawCube(vec4(.8,.6,.3,1));
    mv = mvMatrixStack.popMatrix();
}

//----------------------------------------------------------- display

void
display( void )
{
    // clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // calculate and set camera projection parameters:
    mat4  p = camera.calcPerspective();
    glUniformMatrix4fv( projection, 1, GL_TRUE, p );


    mat4 mv;  // modelview matrix

    // Draw a cube which is *always* a distance d in front of the camera.
    // This cube is for reference only.  TUMBLE rotations can be about this point.
    // Do you see why the cube needs to be drawn *before* we move and orient the camera (i.e. before the View is set).
    mv = mv*Translate(0,0,-camera.d);
    mv = mv*Scale(.5);
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv  );
    shapes.drawCube(vec4(1,1,1,1));
    axes.draw(mv,0);  // draw coordinate axis that are centered on the reference cube
    // End: reference cube

    // Initialize the modelview matrix to contain the camera orientation and location
    mvMatrixStack.loadIdentity();
    // SET THE CAMERA VIEW MATRIX:
    //  NEED TO IMPLEMENT !!!  Initialize mv with V = view matrix
    // mv =  V = viewRotation times EyeTranslation


    axes.draw(mv,1);
    drawGround(mv);
    mv = mv*Scale(.5);
    glUniformMatrix4fv( model_view, 1, GL_TRUE, mv );
    myCar.draw(mv);

    glBindVertexArray( 0 );
    glutSwapBuffers();
}


//---------------------------------------------------------------------------- keyboard

void
keyboard( unsigned char key, int x, int y )
{
    switch( key )
    {
    case 033: // Escape Key
    case 'q':
    case 'Q':
        exit( EXIT_SUCCESS );
        break;
    case 'f':     // drive car forward
        myCar.wheelAngle += 5;
        myCar.xLoc -= 2*M_PI*2.*5/360.;
        break;
    case 'b':     // drive car forward
        myCar.wheelAngle -= 5;
        myCar.xLoc += 2*M_PI*2.*5/360.;
        break;
    case 't':     // toggle tumblepoint
        camera.toggle();
        break;
    case 'r':     // reset
        camera.reset();
        break;
    }

    glutPostRedisplay();
}

void keySpecial( int key, int x, int y )
{
    camera.keySpecial(key,x,y);
}
//---------------------------------------------------------------------------- motion, mouse
// Mouse controls
void
motion( GLint x, GLint y )
{
    camera.motion(x,y);
}

void
mouse( GLint button, GLint state, GLint x, GLint y )
{
    camera.mouseControl(button,state,x,y);
}

//---------------------------------------------------------------------------- reshape

void
reshape( int width, int height )
{
    glViewport( 0, 0, width, height );
    camera.aspect = GLfloat(width)/height;
}

void idle()
{
    animateAngle += 1;
    if (animateAngle <0) animateAngle = 0;

    glutPostRedisplay();
}

//---------------------------------------------------------------------------- main

int
main( int argc, char **argv )
{
    glutInit( &argc, argv );
    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    glutInitWindowSize( 512, 512 );
    glutInitContextVersion( 3, 2 );
    glutInitContextProfile( GLUT_CORE_PROFILE );
    glutCreateWindow( "Navigation Example" );

    glewInit();

    init();

    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    glutSpecialFunc(keySpecial);
    glutMouseFunc( mouse );
    glutMotionFunc( motion );
    glutReshapeFunc( reshape );
    // glutIdleFunc(idle); //  need for animation but not mouse interaction

    glutMainLoop();
    return 0;
}
